@@ -207,7 +207,7 @@ clock_adjust(void)
 		double *value = statx_data(datapoints);
 
 		if (GNET_PROPERTY(clock_debug) > 1)
-			g_debug("CLOCK before #%d: n=%d avg=%.2f sdev=%.2f",
+			g_debug("CLOCK before #%d: n=%d avg=%g sdev=%g",
 				k, n, avg, sdev);
 
 		statx_clear(datapoints);
@@ -238,7 +238,7 @@ clock_adjust(void)
 		sdev = statx_sdev(datapoints);
 
 		if (GNET_PROPERTY(clock_debug) > 1)
-			g_debug("CLOCK after #%d: kept n=%d avg=%.2f sdev=%.2f",
+			g_debug("CLOCK after #%d: kept n=%d avg=%g sdev=%g",
 				k, n, avg, sdev);
 
 		if (sdev <= MAX_SDEV || n < MIN_DATA)
@@ -264,7 +264,7 @@ clock_adjust(void)
 	new_skew = GNET_PROPERTY(clock_skew) + (gint32) avg;
 
 	if (GNET_PROPERTY(clock_debug))
-		g_debug("CLOCK with n=%d avg=%.2f sdev=%.2f => SKEW old=%d new=%d",
+		g_debug("CLOCK with n=%d avg=%g sdev=%g => SKEW old=%d new=%d",
 			n, avg, sdev, (gint32) GNET_PROPERTY(clock_skew),
 			(gint32) new_skew);
 
@@ -312,7 +312,7 @@ clock_update(time_t update, int precision, const host_addr_t addr)
 	statx_add(datapoints, (double) (delta - precision));
 
 	if (GNET_PROPERTY(clock_debug) > 1)
-		g_debug("CLOCK skew=%d delta=%d +/-%d [%s] (n=%d avg=%.2f sdev=%.2f)",
+		g_debug("CLOCK skew=%d delta=%d +/-%d [%s] (n=%d avg=%g sdev=%g)",
 			(gint32) GNET_PROPERTY(clock_skew),
 			delta, precision, host_addr_to_string(addr),
 			statx_n(datapoints), statx_avg(datapoints), statx_sdev(datapoints));
