@@ -36,13 +36,18 @@
 #include "ggep.h"
 #include "ggep_type.h"
 #include "hosts.h"				/* For struct gnutella_host */
+#include "ipp_cache.h"			/* For tls_cache_lookup() */
+#include "qhit.h"				/* For QHIT_F_* flags */
 
 #include "lib/bstr.h"
 #include "lib/endian.h"
 #include "lib/gnet_host.h"
+#include "lib/log.h"
 #include "lib/misc.h"
+#include "lib/sequence.h"
 #include "lib/unsigned.h"
 #include "lib/utf8.h"
+#include "lib/vector.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -206,7 +211,7 @@ ggept_gtkgv_osname_value(void)
 			if (-1 != uname(&un)) {
 				result = ggept_gtkgv_osname_encode(un.sysname);
 			} else {
-				g_carp("uname() failed: %s", g_strerror(errno));
+				s_carp("uname() failed: %m");
 			}
 		}
 #else
@@ -350,39 +355,365 @@ ggept_gtkgv1_extract(const extvec_t *exv, struct ggep_gtkgv1 *info)
 	return GGEP_OK;
 }
 
+/**
+ * From a sequence of IP:port addresses, fill a set of GGEP extensions:
+ *
+ *		N and N_TLS
+ *
+ * for the given network type.
+ *
+ * @param gs		the GGEP stream to which extensions are written
+ * @param hseq		sequence of IP:port (IPv4 and IPv6 mixed)
+ * @param net		network address type
+ * @param name		name of GGEP extension for addresses
+ * @param name_tls	name of GGEP extension for TLS
+ * @param evec		optional exclusion address vector
+ * @param ecnt		length of evec[]
+ * @param count		input: max amount of items to generate, output: items sent
+ * @param cobs		whether COBS encoding is required
+ *
+ * @return TRUE on success, FALSE on write errors.
+ */
+static gboolean
+ggept_ip_seq_append_net(ggep_stream_t *gs,
+	const sequence_t *hseq, enum net_type net,
+	const char *name, const char *name_tls,
+	const gnet_host_t *evec, size_t ecnt, size_t *count, gboolean cobs)
+{
+	guchar *tls_bytes = NULL;
+	unsigned tls_length;
+	size_t tls_size = 0, tls_index = 0;
+	gboolean status = FALSE;
+	unsigned flags = 0;
+	size_t hcnt;
+	const char *current_extension;
+	sequence_iter_t *iter = NULL;
+	size_t max_items = *count;
+
+	g_assert(gs != NULL);
+	g_assert(name != NULL);
+	g_assert(hseq != NULL);
+
+	hcnt = sequence_count(hseq);
+
+	if (0 == hcnt) {
+		status = TRUE;
+		goto done;
+	}
+
+	tls_size = (hcnt + 7) / 8;
+	tls_bytes = name_tls != NULL ? walloc0(tls_size) : NULL;
+	tls_index = tls_length = 0;
+
+	/*
+	 * We only attempt to deflate IPv6 vectors, since IPv4 does not bring
+	 * enough redundancy to be worth it: 180 bytes of data for 30 IPv4
+	 * addresses typically compress to 175 bytes.  Hardly interesting.
+	 */
+
+	flags |= (NET_TYPE_IPV6 == net) ? GGEP_W_DEFLATE : 0;
+	flags |= cobs ? GGEP_W_COBS : 0;
+
+	/*
+	 * We use GGEP_W_STRIP to make sure the extension is removed if empty.
+	 */
+
+	current_extension = name;
+
+	if (!ggep_stream_begin(gs, name, GGEP_W_STRIP | flags))
+		goto done;
+
+	iter = sequence_forward_iterator(hseq);
+
+	while (sequence_iter_has_next(iter) && tls_index < max_items) {
+		host_addr_t addr;
+		guint16 port;
+		char buf[18];
+		size_t len;
+		const gnet_host_t *h = sequence_iter_next(iter);
+
+		if (net != gnet_host_get_net(h))
+			continue;
+
+		/*
+		 * See whether we need to skip that host.
+		 */
+
+		if (evec != NULL) {
+			size_t i;
+
+			for (i = 0; i < ecnt; i++) {
+				if (gnet_host_eq(h, &evec[i]))
+					goto next;
+			}
+		}
+
+		addr = gnet_host_get_addr(h);
+		port = gnet_host_get_port(h);
+
+		host_ip_port_poke(buf, addr, port, &len);
+		if (!ggep_stream_write(gs, buf, len))
+			goto done;
+
+		/*
+		 * Record in bitmask whether host is known to support TLS.
+		 */
+
+		if (name_tls != NULL && tls_cache_lookup(addr, port)) {
+			tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
+			tls_length = (tls_index >> 3) + 1;
+		}
+		tls_index++;
+	next:
+		continue;
+	}
+
+	if (!ggep_stream_end(gs))
+		goto done;
+
+	if (tls_length > 0) {
+		unsigned gflags = cobs ? GGEP_W_COBS : 0;
+		g_assert(name_tls != NULL);
+		current_extension = name_tls;
+		if (!ggep_stream_pack(gs, name_tls, tls_bytes, tls_length, gflags))
+			goto done;
+	}
+
+	status = TRUE;
+
+done:
+	if (!status) {
+		g_carp("unable to add GGEP \"%s\"", current_extension);
+	}
+
+	*count = tls_index;
+
+	sequence_iterator_release(&iter);
+	WFREE_NULL(tls_bytes, tls_size);
+	return status;
+}
+
+/**
+ * From a sequence of IP:port addresses fill two sets of GGEP extensions:
+ *
+ *    NAME and NAME_TLS for IPv4 addresses
+ *    NAME6 and NAME6_TLS for IPv6 addresses
+ *
+ * @param gs		the GGEP stream to which extensions are written
+ * @param hseq		sequence of IP:port (IPv4 and IPv6 mixed)
+ * @param name		name of GGEP extension for IPv4 addresses
+ * @param name_tls	name of GGEP extension for TLS vector of IPv4 addresses
+ * @param name6		name of GGEP extension for IPv6 addresses
+ * @param name6_tls	name of GGEP extension for TLS vector of IPv6 addresses
+ * @param evec		optional exclusion address vector
+ * @param ecnt		length of evec[]
+ * @param max_items	maximum amount of items to include, (size_t) -1 means all
+ * @param cobs		whether COBS encoding is required
+ *
+ * @return GGEP_OK on success, GGEP_BAD_SIZE on write errors.
+ */
 static ggept_status_t
-ggept_ip_vec_extract(const extvec_t *exv, gnet_host_vec_t **hvec)
+ggept_ip_seq_append(ggep_stream_t *gs,
+	const sequence_t *hseq,
+	const char *name, const char *name_tls,
+	const char *name6, const char *name6_tls,
+	const gnet_host_t *evec, size_t ecnt, size_t max_items, gboolean cobs)
+{
+	size_t count = max_items;
+
+	if (name != NULL && count != 0) {
+		if (
+			!ggept_ip_seq_append_net(gs, hseq, NET_TYPE_IPV4,
+				name, name_tls, evec, ecnt, &count, cobs)
+		) {
+			return GGEP_BAD_SIZE;
+		}
+	}
+
+	g_assert(count <= max_items);
+	count = max_items - count;
+
+	if (name6 != NULL && count != 0) {
+		if (
+			!ggept_ip_seq_append_net(gs, hseq, NET_TYPE_IPV6,
+				name6, name6_tls, evec, ecnt, &count, cobs)
+		) {
+			return GGEP_BAD_SIZE;
+		}
+	}
+	return GGEP_OK;
+}
+
+/**
+ * Emit vector of IP:port addresses for "IPP".
+ *
+ * @param gs		the GGEP stream to which extensions are written
+ * @param hvec		vector of IP:port (IPv4 and IPv6 mixed)
+ * @param hcnt		length of hvec[]
+ * @param evec		exclusion address vector
+ * @param ecnt		length of evec[]
+ * @param add_ipv6	whether IPv6 addresses are requested
+ * @param no_ipv4	whether IPv4 addresses should be excluded
+ */
+ggept_status_t
+ggept_ipp_pack(ggep_stream_t *gs, const gnet_host_t *hvec, size_t hcnt,
+	const gnet_host_t *evec, size_t ecnt,
+	gboolean add_ipv6, gboolean no_ipv4)
+{
+	vector_t v = vector_create(deconstify_gpointer(hvec), sizeof *hvec, hcnt);
+	sequence_t hseq;
+
+	sequence_fill_from_vector(&hseq, &v);
+
+	return ggept_ip_seq_append(gs, &hseq,
+		no_ipv4 ? NULL : GGEP_NAME(IPP), GGEP_NAME(IPP_TLS),
+		add_ipv6 ? GGEP_NAME(IPP6) : NULL, GGEP_NAME(IPP6_TLS),
+		evec, ecnt, (size_t) -1, FALSE);
+}
+
+/**
+ * Emit vector of IP:port addresses for "DHTIPP".
+ *
+ * @param gs		the GGEP stream to which extensions are written
+ * @param hvec		vector of IP:port (IPv4 and IPv6 mixed)
+ * @param hcnt		length of hvec[]
+ * @param add_ipv6	whether IPv6 addresses are requested
+ * @param no_ipv4	whether IPv4 addresses should be excluded
+ */
+ggept_status_t
+ggept_dhtipp_pack(ggep_stream_t *gs, const gnet_host_t *hvec, size_t hcnt,
+	gboolean add_ipv6, gboolean no_ipv4)
+{
+	vector_t v = vector_create(deconstify_gpointer(hvec), sizeof *hvec, hcnt);
+	sequence_t hseq;
+
+	sequence_fill_from_vector(&hseq, &v);
+
+	return ggept_ip_seq_append(gs, &hseq,
+		no_ipv4 ? NULL : GGEP_NAME(IPP), NULL,
+		add_ipv6 ? GGEP_NAME(IPP6) : NULL, NULL,
+		NULL, 0, (size_t) -1, FALSE);
+}
+
+/**
+ * Emit sequence of IP:port addresses for "PUSH".
+ *
+ * @param gs		the GGEP stream to which extensions are written
+ * @param hseq		sequence of IP:port (IPv4 and IPv6 mixed)
+ * @param max		maximum amount of entries to add
+ * @param flags		a combination of QHIT_F_* flags
+ */
+ggept_status_t
+ggept_push_pack(ggep_stream_t *gs, const sequence_t *hseq, size_t max,
+	unsigned flags)
+{
+	return ggept_ip_seq_append(gs, hseq,
+		(flags & QHIT_F_IPV6_ONLY) ? NULL : GGEP_NAME(PUSH),
+		GGEP_NAME(PUSH_TLS),
+		(flags & QHIT_F_IPV6) ? GGEP_NAME(PUSH6) : NULL, GGEP_NAME(PUSH6_TLS),
+		NULL, 0, max, FALSE);
+}
+
+/**
+ * Emit sequence of IP:port addresses for "A" in HEAD pongs.
+ *
+ * @param gs		the GGEP stream to which extensions are written
+ * @param hvec		vector of IP:port (IPv4 and IPv6 mixed)
+ * @param hcnt		length of hvec[]
+ */
+ggept_status_t
+ggept_a_pack(ggep_stream_t *gs, const gnet_host_t *hvec, size_t hcnt)
+{
+	vector_t v = vector_create(deconstify_gpointer(hvec), sizeof *hvec, hcnt);
+	sequence_t hseq;
+
+	sequence_fill_from_vector(&hseq, &v);
+
+	return ggept_ip_seq_append(gs, &hseq,
+		GGEP_NAME(A), GGEP_NAME(T),
+		GGEP_NAME(A6), GGEP_NAME(T6),
+		NULL, 0, (size_t) -1, FALSE);
+}
+
+/**
+ * Emit sequence of IP:port addresses for "ALT" in query hits.
+ *
+ * @param gs		the GGEP stream to which extensions are written
+ * @param hvec		vector of IP:port (IPv4 and IPv6 mixed)
+ * @param hcnt		length of hvec[]
+ * @param flags		a combination of QHIT_F_* flags
+ */
+ggept_status_t
+ggept_alt_pack(ggep_stream_t *gs, const gnet_host_t *hvec, size_t hcnt,
+	unsigned flags)
+{
+	vector_t v = vector_create(deconstify_gpointer(hvec), sizeof *hvec, hcnt);
+	sequence_t hseq;
+
+	sequence_fill_from_vector(&hseq, &v);
+
+	/* This needs COBS encoding */
+
+	return ggept_ip_seq_append(gs, &hseq,
+		(flags & QHIT_F_IPV6_ONLY) ? NULL : GGEP_NAME(ALT), GGEP_NAME(ALT_TLS),
+		(flags & QHIT_F_IPV6) ? GGEP_NAME(ALT6) : NULL, GGEP_NAME(ALT6_TLS),
+		NULL, 0, (size_t) -1, TRUE);
+}
+
+static ggept_status_t
+ggept_ip_vec_extract(const extvec_t *exv,
+	gnet_host_vec_t **hvec, enum net_type net)
 {
 	int len;
+	int ilen;
 
 	g_assert(exv);
 	g_assert(hvec);
 	g_assert(EXT_GGEP == exv->ext_type);
+	g_assert(NET_TYPE_IPV4 == net || NET_TYPE_IPV6 == net);
 
 	len = ext_paylen(exv);
+	ilen = NET_TYPE_IPV4 == net ? 6 : 18;	/* IP + port */
 
 	if (len <= 0)
 		return GGEP_INVALID;
 
-	if (len % 6 != 0)
+	if (len % ilen != 0)
 		return GGEP_INVALID;
 
 	if (hvec) {
-		gnet_host_vec_t *vec;
+		gnet_host_vec_t *vec = *hvec;
 		const char *p;
 		guint n, i;
 
-		vec = gnet_host_vec_alloc();
-		n = len / 6;
-		n = MIN(n, 255);	/* n_ipv4 is guint8 */
-		vec->n_ipv4 = n;
-		vec->hvec_v4 = walloc(n * sizeof vec->hvec_v4[0]);
+		vec = NULL == vec ? gnet_host_vec_alloc() : vec;
+		n = len / ilen;
+		n = MIN(n, 255);	/* n_ipv4 and n_ipv6 are guint8 */
+
+		g_assert(n > 0);
+
+		if (NET_TYPE_IPV4 == net) {
+			if (vec->n_ipv4 != 0)
+				return GGEP_DUPLICATE;
+			vec->n_ipv4 = n;
+			vec->hvec_v4 = walloc(n * sizeof vec->hvec_v4[0]);
+		} else {
+			if (vec->n_ipv6 != 0)
+				return GGEP_DUPLICATE;
+			vec->n_ipv6 = n;
+			vec->hvec_v6 = walloc(n * sizeof vec->hvec_v6[0]);
+		}
 
 		p = ext_payload(exv);
 		for (i = 0; i < n; i++) {
-			/* IPv4 address (BE) + Port (LE) */
-			memcpy(&vec->hvec_v4[i].data, p, 6);
-			p += 6;
+			/* IPv4 address (BE) or IPv6 address (BE) + Port (LE) */
+			if (NET_TYPE_IPV4 == net) {
+				memcpy(&vec->hvec_v4[i].data, p, 6);
+				p += 6;
+			} else {
+				memcpy(&vec->hvec_v6[i].data, p, 18);
+				p += 18;
+			}
 		}
 		*hvec = vec;
 	}
@@ -393,34 +724,52 @@ ggept_ip_vec_extract(const extvec_t *exv, gnet_host_vec_t **hvec)
 /**
  * Extract vector of IP:port alternate locations.
  *
- * @param `exv'		no brief description.
- * @param `hvec'	pointer is filled with a dynamically allocated vector.
- *
+ * The `hvec' pointer is filled with a dynamically allocated vector.
  * Unless GGEP_OK is returned, no memory allocation takes place.
+ *
+ * If *hvec is not NULL, it is filled with new hosts provided that there were
+ * no hosts for that kind yet within the vector.
+ *
+ * @param exv	the extension vector
+ * @param hvec	pointer is filled with a dynamically allocated vector.
+ * @param net	type of network addresses expected in the extension
+ *
+ * @return GGEP_OK on success
  */
 ggept_status_t
-ggept_alt_extract(const extvec_t *exv, gnet_host_vec_t **hvec) 
+ggept_alt_extract(const extvec_t *exv,
+	gnet_host_vec_t **hvec, enum net_type net) 
 {
 	g_assert(exv->ext_type == EXT_GGEP);
-	g_assert(exv->ext_token == EXT_T_GGEP_ALT);
+	g_assert(exv->ext_token == EXT_T_GGEP_ALT ||
+		exv->ext_token == EXT_T_GGEP_ALT6);
 
-	return ggept_ip_vec_extract(exv, hvec);
+	return ggept_ip_vec_extract(exv, hvec, net);
 }
 
 /**
  * Extract vector of IP:port push proxy locations.
  *
  * The `hvec' pointer is filled with a dynamically allocated vector.
- *
  * Unless GGEP_OK is returned, no memory allocation takes place.
+ *
+ * If *hvec is not NULL, it is filled with new hosts provided that there were
+ * no hosts for that kind yet within the vector.
+ *
+ * @param exv	the extension vector
+ * @param hvec	pointer is filled with a dynamically allocated vector.
+ * @param net	type of network addresses expected in the extension
+ *
+ * @return GGEP_OK on success
  */
 ggept_status_t
-ggept_push_extract(const extvec_t *exv, gnet_host_vec_t **hvec)
+ggept_push_extract(const extvec_t *exv,
+	gnet_host_vec_t **hvec, enum net_type net)
 {
 	g_assert(exv->ext_type == EXT_GGEP);
 	g_assert(exv->ext_token == EXT_T_GGEP_PUSH);
 
-	return ggept_ip_vec_extract(exv, hvec);
+	return ggept_ip_vec_extract(exv, hvec, net);
 }
 
 /**
@@ -566,9 +915,8 @@ ggept_filesize_extract(const extvec_t *exv, guint64 *filesize)
 }
 
 /**
- * Extract IPv6 address into `addr' from GGEP "GTKG.IPV6" extension.
- * A zero length indicates IPv6 support, a length of 16 or more
- * indicates that the first 16 bytes are a IPv6 address.
+ * Extract IPv6 address into `addr' from GGEP "GTKG.IPV6" or "6" extensions.
+ * When "addr" is NULL, simply validates the payload length.
  */
 ggept_status_t
 ggept_gtkg_ipv6_extract(const extvec_t *exv, host_addr_t *addr)
@@ -576,13 +924,14 @@ ggept_gtkg_ipv6_extract(const extvec_t *exv, host_addr_t *addr)
 	size_t len;
 
 	g_assert(exv->ext_type == EXT_GGEP);
-	g_assert(exv->ext_token == EXT_T_GGEP_GTKG_IPV6);
+	g_assert(exv->ext_token == EXT_T_GGEP_GTKG_IPV6 ||
+		exv->ext_token == EXT_T_GGEP_6);
 
 	len = ext_paylen(exv);
-	if (0 != len && 16 < len)
+	if (0 != len && len < 16)
 		return GGEP_INVALID;
 
-	if (addr) {
+	if (addr != NULL) {
 		if (0 == len) {
 			*addr = zero_host_addr;
 		} else {
