@@ -1,6 +1,4 @@
 /*
- * $Id: namesize.h 8376 2005-06-25 01:37:45Z daichik $
- *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -36,14 +34,14 @@
 #ifndef _core_namesize_h_
 #define _core_namesize_h_
 
-#include <glib.h>
+#include "common.h"
 
 /**
  * The (name, size) tuple.
  */
 
 typedef struct namesize {
-	gchar *name;		/**< Filename (atom) */
+	const char *name;		/**< Filename (atom) */
 	filesize_t size;	/**< File size */
 } namesize_t;
 
@@ -52,9 +50,9 @@ typedef struct namesize {
  */
 
 guint namesize_hash(gconstpointer key);
-gint namesize_eq(gconstpointer a, gconstpointer b);
+int namesize_eq(gconstpointer a, gconstpointer b);
 
-namesize_t *namesize_make(const gchar *name, filesize_t size);
+namesize_t *namesize_make(const char *name, filesize_t size);
 void namesize_free(namesize_t *ns);
 
 #endif	/* _core_namesize_h_ */
