@@ -108,8 +108,8 @@ spam_lut_create(void)
 
 		if (NULL == dm) {
 			if (GNET_PROPERTY(spam_debug))
-				g_warning("unable to create SDBM database for %s: %s",
-					db_spambase, g_strerror(errno));
+				g_warning("unable to create SDBM database for %s: %m",
+					db_spambase);
 			sha1_lut.tab = sorted_array_new(sizeof(struct sha1), sha1_cmp_func);
 		} else {
 			/*
@@ -147,7 +147,7 @@ static int
 sha1_collision(const void *a, const void *b)
 {
 	(void) a;
-	g_warning("spam_sha1_sync(): Removing duplicate SHA-1 %s", sha1_base32(b));
+	g_warning("spam_sha1_sync(): removing duplicate SHA-1 %s", sha1_base32(b));
 	return 1;
 }
 
@@ -199,32 +199,29 @@ spam_sha1_load(FILE *f)
 
 	while (fgets(line, sizeof line, f)) {
 		const struct sha1 *sha1;
-		char *nl;
+		size_t len;
 
 		line_no++;
 
-		nl = strchr(line, '\n');
-		if (!nl) {
+		if (!file_line_chomp_tail(line, sizeof line, &len)) {
 			/*
 			 * If the line is too long or unterminated the file is either
 			 * corrupt or was manually edited without respecting the
 			 * exact format. If we continued, we would read from the
 			 * middle of a line which could be the filename or ID.
 			 */
-			g_warning("spam_sha1_load(): "
-				"line too long or missing newline in line %u",
-				line_no);
+			g_warning("%s(): line %u too long or missing newline",
+				G_STRFUNC, line_no);
 			break;
 		}
-		*nl = '\0';
 
 		/* Skip comments and empty lines */
-		if (*line == '#' || *line == '\0')
+		if (file_line_is_skipable(line))
 			continue;
 
-		if (strlen(line) < SHA1_BASE32_SIZE) {
-			g_warning("spam_sha1_load(): SHA-1 has wrong length in line %u.",
-				line_no);
+		if (len < SHA1_BASE32_SIZE) {
+			g_warning("%s(): SHA-1 has wrong length %zu in line %u.",
+				G_STRFUNC, len, line_no);
 			continue;
 		}
 
@@ -233,15 +230,14 @@ spam_sha1_load(FILE *f)
 		 * the leading SHA-1 is separated from the trailing data.
 		 */
 		if (is_ascii_alnum(line[SHA1_BASE32_SIZE])) {
-			g_warning("spam_sha1_load(): Bad SHA-1 in line %u.",
-				line_no);
+			g_warning("%s(): bad SHA-1 in line %u.", G_STRFUNC, line_no);
 			continue;
 		}
 		
 		sha1 = base32_sha1(line);
 		if (NULL == sha1) {
-			g_warning("spam_sha1_load(): Could not parse SHA-1 in line %u.",
-				line_no);
+			g_warning("%s(): could not parse SHA-1 in line %u.",
+				G_STRFUNC, line_no);
 			continue;
 		}
 		spam_sha1_add(sha1);
@@ -323,6 +319,8 @@ spam_sha1_retrieve(void)
 	if (tmp != NULL)
 		file_path_set(&fp[length++], tmp, spam_sha1_file);
 
+	g_assert(length <= G_N_ELEMENTS(fp));
+
 	f = file_config_open_read_norename_chosen(spam_sha1_what, fp, length, &idx);
 	if (f != NULL) {
 		spam_sha1_retrieve_from_file(f, fp[idx].dir, fp[idx].name);
