@@ -33,8 +33,23 @@
 #ifdef HAS_GNUTLS
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
+
+#ifdef GNUTLS_VERSION_MAJOR
+#define HAS_TLS(major, minor) \
+	((GNUTLS_VERSION_MAJOR > (major) || \
+	 (GNUTLS_VERSION_MAJOR == (major) && GNUTLS_VERSION_MINOR >= (minor))))
+#else
+#define HAS_TLS(major, minor) 0
+#endif
+
+#else
+#define HAS_TLS(major, minor) 0
 #endif /* HAS_GNUTLS */
 
+#if HAS_TLS(2, 12)
+#include <gnutls/abstract.h>
+#endif
+
 #include "tls_common.h"
 #include "features.h"
 #include "sockets.h"
@@ -88,26 +103,21 @@ static inline void
 tls_transport_debug(const char *op, const struct gnutella_socket *s,
 	size_t size, ssize_t ret)
 {
-	int saved_errno = errno;
-
 	if ((ssize_t) -1 == ret) {
-		unsigned level = is_temporary_error(saved_errno) ? 2 : 0;
+		unsigned level = is_temporary_error(errno) ? 2 : 0;
 
 		if (GNET_PROPERTY(tls_debug) > level) {
-			g_debug("%s(): fd=%d size=%lu host=%s ret=-1 errno=%s",
-				op, s->file_desc, (unsigned long) size,
-				host_addr_port_to_string(s->addr, s->port),
-				symbolic_errno(saved_errno));
+			g_debug("%s(): fd=%d size=%zu host=%s ret=-1 errno=%m",
+				op, s->file_desc, size,
+				host_addr_port_to_string(s->addr, s->port));
 		}
 	} else {
 		if (GNET_PROPERTY(tls_debug) > 2) {
-			g_debug("%s(): fd=%d size=%lu host=%s ret=%lu",
-				op, s->file_desc, (unsigned long) size,
-				host_addr_port_to_string(s->addr, s->port),
-				(unsigned long) ret);
+			g_debug("%s(): fd=%d size=%zu host=%s ret=%zu",
+				op, s->file_desc, size,
+				host_addr_port_to_string(s->addr, s->port), ret);
 		}
 	}
-	errno = saved_errno;
 }
 
 /**
@@ -147,7 +157,7 @@ tls_signal_pending(struct gnutella_socket *s)
 		int saved_errno = errno;
 
 		if (GNET_PROPERTY(tls_debug) > 1) {
-			g_debug("%s: pending=%lu", G_STRFUNC, (unsigned long) n);
+			g_debug("%s: pending=%zu", G_STRFUNC, n);
 		}
 		inputevt_set_readable(s->file_desc);
 		errno = saved_errno;
@@ -394,8 +404,18 @@ tls_init(struct gnutella_socket *s)
 	gnutls_transport_set_ptr(ctx->session, s);
 	gnutls_transport_set_push_function(ctx->session, tls_push);
 	gnutls_transport_set_pull_function(ctx->session, tls_pull);
+#if !HAS_TLS(2, 12)
+	/*
+	 * This routine has been removed starting TLS 3.0.  It was used to disable
+	 * the lowat feature, and apparently this is now always the case in recent
+	 * TLS versions.	--RAM, 2011-09-28
+	 *
+	 * It's also flagged as deprecated in 2.12.x, so don't use it there.
+	 *		--RAM, 2011-12-15
+	 */
 	gnutls_transport_set_lowat(ctx->session, 0);
-#else
+#endif
+#else	/* !USE_TLS_CUSTOM_IO */
 	g_assert(is_valid_fd(s->file_desc));
 	gnutls_transport_set_ptr(ctx->session, int_to_pointer(s->file_desc));
 #endif	/* USE_TLS_CUSTOM_IO */
@@ -557,10 +577,9 @@ tls_write_intern(struct wrap_io *wio, const void *buf, size_t size)
 		default:
 			if (GNET_PROPERTY(tls_debug)) {
 				g_carp("tls_write(): gnutls_record_send(fd=%d) failed: "
-					"host=%s snarf=%lu error=\"%s\"",
+					"host=%s snarf=%zu error=\"%s\"",
 					s->file_desc, host_addr_port_to_string(s->addr, s->port),
-					(unsigned long) s->tls.snarf,
-					gnutls_strerror(ret));
+					s->tls.snarf, gnutls_strerror(ret));
 			}
 			errno = EIO;
 			ret = -1;
@@ -595,8 +614,8 @@ tls_flush(struct wrap_io *wio)
 
 	if (s->tls.snarf) {
 		if (GNET_PROPERTY(tls_debug > 1)) {
-			g_debug("tls_flush: snarf=%lu host=%s fd=%d",
-					(unsigned long) s->tls.snarf,
+			g_debug("tls_flush: snarf=%zu host=%s fd=%d",
+					s->tls.snarf,
 					host_addr_port_to_string(s->addr, s->port), s->file_desc);
 		}
 		(void ) tls_write_intern(wio, NULL, 0);
@@ -643,8 +662,7 @@ tls_read(struct wrap_io *wio, void *buf, size_t size)
 
 	if (tls_flush(wio) && !is_temporary_error(errno)) {
 		if (GNET_PROPERTY(tls_debug)) {
-			g_warning("tls_read: tls_flush(fd=%d) error: %s",
-				s->file_desc, symbolic_errno(errno));
+			g_warning("tls_read: tls_flush(fd=%d) error: %m", s->file_desc);
 		}
 		return -1;
 	}
@@ -806,9 +824,8 @@ tls_bye(struct gnutella_socket *s)
 			break;
 		default:
 			if (GNET_PROPERTY(tls_debug)) {
-				g_carp("gnutls_bye() failed: host=%s error=%s",
-					host_addr_port_to_string(s->addr, s->port),
-					symbolic_errno(errno));
+				g_carp("gnutls_bye() failed: host=%s error=%m",
+					host_addr_port_to_string(s->addr, s->port));
 			}
 		}
 	}
