@@ -55,6 +55,8 @@
 #define LEVEL_SIZE			(2 * G_N_ELEMENTS(token_keys))	/**< at most */
 #define LEVEL_BASE64_SIZE	(LEVEL_SIZE * 4 / 3 + 3)	/**< +2 for == tail */
 
+#define GIT_SWITCH			1315692000		/* 2011-09-11 */
+
 /*
  * Keys are generated through "od -x /dev/random".
  * There can be up to 2^5 = 32 keys per version.
@@ -366,6 +368,71 @@ static const char *keys_097_1[] = {
 	"0c93 3ee1 f48f 3653 7a9c f338 8d2e 8868",
 };
 
+static const char *keys_098_0[] = {
+	"3248 b31b 8cf3 763c 552a d42c d833 1d79",
+	"9dbb 43d8 c217 0ecf 743d 4ac4 02ec 9aed",
+	"4165 dbb9 2afb 61d9 9a7e d13c b65a 05e2",
+	"4971 e904 c448 9802 2eb6 b9e4 d177 83c2",
+	"0bbe a4cb 17be 4427 cad0 2d48 c39f 5464",
+	"110b 8553 bd5d a1da ae75 d863 c652 3d37",
+	"7c19 9b64 b003 55c9 391e 8740 5697 035b",
+	"3e5d f6de fb0e 72aa c25b a9f4 5295 78e8",
+	"e681 10b1 8958 7b54 428b 8c50 1c5b 7c5c",
+	"7f74 ee30 0392 a474 3cdb 7d23 47dd 9956",
+	"6751 4a3c 3a4d 2889 edb2 23fb 14c4 6b8d",
+	"3efa ba6b 0d92 a890 0ee4 676d 0ec6 8884",
+	"6a4b 0ed9 6ca4 771e 2fd9 c14b ea65 f661",
+	"aa57 742b 6ea7 843b 08a4 8228 fcc1 ff1f",
+	"4e42 81bf 8e15 54f5 62bc d970 e456 2345",
+	"194d 052b 25b5 0731 14e8 372a 952a 8535",
+	"1af5 7651 8c7b 8aed ee9d a752 167e d735",
+	"40a8 a0f0 8432 30ba d2ba d297 9b81 6542",
+	"c761 e6cd 1829 8056 7bb0 ffab 6fee 4867",
+	"a498 e32b d8ff 954d f68c 1bfd 6d65 9ff2",
+	"ffb0 3a61 6795 6b47 f7ef 7c9c d08b 913c",
+	"8f62 d747 2f8c 9a2f 019e 3bc0 9c62 7374",
+	"fd79 60f5 9ac7 f1e9 3cde c3c8 8cad bd36",
+	"01a9 c262 e5e5 0c10 fabb d81c 54b9 e9f0",
+	"764c 89cb 4a1b 978f 2217 ec5d 38d7 33f7",
+	"e3b6 8589 dfd1 829a 41f3 b371 0555 80bb",
+	"5376 c257 21cb 0f2b 2f56 f1f3 a772 afd0",
+};
+
+static const char *keys_098_1[] = {
+	"64e2 7991 d384 b6bf 2bbe 7741 152c 8b93",
+	"0ba4 8379 63d4 e789 575c 9035 e3a4 93be",
+	"dfb4 08fe ec3a 2aa3 6018 37be 2a68 06d0",
+	"3e31 23c5 526f 564b 8da9 5315 687e d437",
+	"0728 cfa5 43af e2ae 46ed d9b0 af49 a64a",
+	"2fa9 fff7 57d9 5b5a 46af b557 e388 7826",
+	"b8a5 0059 6464 87a5 dba1 79d1 373f 5966",
+	"150e 70ac 19c8 1040 912e 5cc4 7d37 db12",
+	"3ab0 72d9 9436 4be3 3bea 77fe 409f 3e37",
+	"f5ed 6af6 9fa1 c9af 3ea0 d652 2ac0 5d75",
+	"6a9a 0d02 2abc a36d 0cc8 ae53 8292 c608",
+	"4284 5952 4a2c 8399 cbaa ade3 e320 0712",
+	"9aec 465a f6ec 175a e384 1d22 8570 56be",
+	"234f f3cc c451 cead 33df 5e5d dce7 e492",
+	"8b42 74bc 7703 492c 5ba2 83f5 65eb 4ed3",
+	"9991 5c43 5400 48d4 fab3 65a1 33e0 c102",
+	"e280 ca7d 0143 f98a 3131 4031 a147 b16c",
+	"0211 f8b4 aa81 e9c6 5d93 f9e1 e853 7ca7",
+	"eaf5 5569 653a a80d a730 d528 7512 7bff",
+	"540e b603 96b8 db6b 0a71 2d80 b44f 524f",
+	"98c8 6092 0929 30fb 63bb 177f ebf4 9db9",
+	"cf06 0c7a a6ac 0ed9 9c4c bae3 3a63 3075",
+	"5743 93c2 b5d2 9be4 7fa0 62e2 c3e9 6f22",
+	"d6eb 0f97 2792 e548 aac0 34d5 26e2 4cb2",
+	"b8e6 d222 2e81 e91a ec37 63d0 471e 17b4",
+	"e1ed 3bb0 0efc a215 b3af 560f cd50 fcdf",
+	"7827 f3ef ac6d 92cb aa1e 5696 33a1 2c66",
+	"8091 baf1 4b68 dd57 244b 644b 6639 5ede",
+	"e3bd 34aa b8aa 4bf8 2aa3 0886 f8a7 f645",
+	"e6c7 79b9 a976 7d3b a542 f538 a35f afd7",
+	"8dd7 e6f4 a16a 6287 ccff 1aa8 8a4c 61aa",
+	"d3c8 ed48 c580 8e04 cf55 14f0 a97a d084",
+};
+
 /**
  * Describes the keys to use depending on the version.
  */
@@ -421,9 +488,17 @@ struct tokkey {
 		keys_097_0, G_N_ELEMENTS(keys_097_0),
 	},
 	{
-		{ 0, 97, 1, '\0', 0, 0, 1315692000 },		/* 2011-09-11 */
+		{ 0, 97, 1, '\0', 0, 0, GIT_SWITCH },		/* 2011-09-11 */
 		keys_097_1, G_N_ELEMENTS(keys_097_1),
 	},
+	{
+		{ 0, 98, 0, '\0', 0, 0, 1323558000 },		/* 2011-12-11 */
+		keys_098_0, G_N_ELEMENTS(keys_098_0),
+	},
+	{
+		{ 0, 98, 1, '\0', 0, 0, 1323990000 },		/* 2011-12-16 */
+		keys_098_1, G_N_ELEMENTS(keys_098_1),
+	},
 };
 
 /**
@@ -476,8 +551,8 @@ find_tokkey_upto(time_t now, size_t count)
 	guint i;
 
 	if (GNET_PROPERTY(version_debug) > 4) {
-		g_debug("%s: count=%lu, from %s()",
-			G_STRFUNC, (unsigned long) count, stacktrace_caller_name(1));
+		g_debug("%s: count=%zu, from %s()",
+			G_STRFUNC, count, stacktrace_caller_name(1));
 	}
 
 	g_assert(count <= G_N_ELEMENTS(token_keys));
@@ -560,9 +635,16 @@ find_tokkey_version(const version_t *ver, time_t now)
 	 * All versions before r16370 used the first key set when they expired.
 	 * If we're more recent, we probably have a stripped list of past key
 	 * sets, and therefore cannot validate their token.
+	 *
+	 * All versions after we switched to git are can be checked provided
+	 * we still have a copy of the keys that they had at the time they
+	 * were released.
 	 */
 
-	if (ver->build < 16370)
+	if (
+		ver->timestamp < GIT_SWITCH &&			/* Before we switched to git */
+		ver->build != 0 && ver->build < 16370
+	)
 		return find_tokkey(now);
 
 	if (GNET_PROPERTY(version_debug) > 4) {
@@ -591,8 +673,8 @@ find_tokkey_version(const version_t *ver, time_t now)
 	}
 
 	if (GNET_PROPERTY(version_debug) > 4) {
-		g_debug("%s: fallback max=%u (/%lu)",
-			G_STRFUNC, i, (unsigned long) G_N_ELEMENTS(token_keys));
+		g_debug("%s: fallback max=%u (/%zu)",
+			G_STRFUNC, i, G_N_ELEMENTS(token_keys));
 	}
 
 	i++;									/* We need a count, not an index */
@@ -891,9 +973,16 @@ tok_version_valid(
 
 	/*
 	 * Verify build.
+	 *
+	 * Build numbers were emitted when we switched to SVN on 2006-08-26
+	 * and stopped being a monotonous increasing function when we switched
+	 * to git on 2011-09-11.
 	 */
 
-	if (rver.timestamp >= 1156543200) {		/* 2006-08-26 */
+	if (
+		rver.timestamp >= 1156543200 &&		/* 2006-08-26 */
+		rver.timestamp < GIT_SWITCH			/* 2011-09-11 */
+	) {
 		if (0 == rver.build)
 			return TOK_MISSING_BUILD;
 		if (rver.build < latest->ver.build)
