@@ -39,11 +39,11 @@
 
 #define GTA_PRODUCT_NAME "gtk-gnutella"	/**< Normally "gtk-gnutella" */
 #define GTA_VERSION 0				/**< major version */
-#define GTA_SUBVERSION 97			/**< minor version */
+#define GTA_SUBVERSION 98			/**< minor version */
 #define GTA_PATCHLEVEL 1			/**< patch level or teeny version */
 #define GTA_REVISION "stable"		/**< unstable, beta, stable */
 #define GTA_REVCHAR ""				/**< (u)nstable, (b)eta, none -> stable */
-#define GTA_RELEASE "2011-09-11"	/**< ISO 8601 format YYYY-MM-DD */
+#define GTA_RELEASE "2011-16-11"	/**< ISO 8601 format YYYY-MM-DD */
 #define GTA_WEBSITE "http://gtk-gnutella.sourceforge.net/"
 
 #if defined(USE_GTK1)
@@ -72,6 +72,14 @@
 #define GTA_BUILD					"$Revision$"
 #endif
 
+/*
+ * GTA_BUILD_DATE comes from "revision.h" and supersedes GTA_RELEASE.
+ */
+#ifdef GTA_BUILD_DATE
+#undef GTA_RELEASE
+#define GTA_RELEASE GTA_BUILD_DATE
+#endif
+
 #endif	/* _gtk_gnutella_h_ */
 
 /* vi: set ts=4: */
