@@ -37,6 +37,7 @@
 #include "fast_assert.h"
 #include "log.h"
 #include "stacktrace.h"
+#include "str.h"
 #include "override.h"			/* Must be the last header included */
 
 /**
@@ -74,41 +75,16 @@ assertion_message(const assertion_data * const data, int fatal)
 		flush_str(STDOUT_FILENO);
 }
 
-/*
- * Due to an optimizer bug in gcc 4.2.1 (and maybe later verions), avoid
- * specifying the REGPARM(1) attribute in the assertion_xxx() routines
- * or the pointer being passed will be garbage, causing a segmentation fault
- * in assertion_message().
- *		--RAM, 2009-10-31
+/**
+ * Abort execution, possibly dumping a stack frame.
  */
-
-NO_INLINE void G_GNUC_COLD NON_NULL_PARAM((1)) /* REGPARM(1) */
-assertion_warning(const assertion_data * const data)
-{
-	assertion_message(data, FALSE);
-
-	/*
-	 * Trace the code path leading to this assertion warning.
-	 */
-
-	stacktrace_where_safe_print_offset(STDERR_FILENO, 1);
-	if (log_stdout_is_distinct())
-		stacktrace_where_safe_print_offset(STDOUT_FILENO, 1);
-}
-
-NO_INLINE void G_GNUC_COLD G_GNUC_NORETURN NON_NULL_PARAM((1)) /* REGPARM(1) */
-assertion_failure(const assertion_data * const data)
+static G_GNUC_COLD G_GNUC_NORETURN void
+assertion_abort(void)
 {
 	static volatile sig_atomic_t seen_fatal;
 
-	assertion_message(data, TRUE);
-
-	/*
-	 * Record the root cause of the assertion failure to be able to log it
-	 * in the crash log in case they don't have gdb available.
-	 */
-
-	crash_assert_failure(data);
+	
+#define STACK_OFF	2		/* 2 extra calls: assertion_failure(), then here */
 
 	/*
 	 * We're going to stop the execution.
@@ -121,9 +97,9 @@ assertion_failure(const assertion_data * const data)
 
 	if (!seen_fatal) {
 		seen_fatal = TRUE;
-		stacktrace_where_cautious_print_offset(STDERR_FILENO, 1);
+		stacktrace_where_cautious_print_offset(STDERR_FILENO, STACK_OFF);
 		if (log_stdout_is_distinct())
-			stacktrace_where_cautious_print_offset(STDOUT_FILENO, 1);
+			stacktrace_where_cautious_print_offset(STDOUT_FILENO, STACK_OFF);
 
 		/*
 		 * Before calling abort(), which will generate a SIGABRT and invoke
@@ -132,10 +108,155 @@ assertion_failure(const assertion_data * const data)
 		 * longer be possible to get the frame of the assertion failure.
 		 */
 
-		crash_save_current_stackframe();
+		crash_save_current_stackframe(STACK_OFF);
 	}
 
-	abort();
+	/*
+	 * We used to call abort() here.
+	 *
+	 * However, assertion handling is already coupled to crash handling and
+	 * therefore it buys us little to call abort() to raise a SIGABRT signal
+	 * which will then be trapped by the crash handler anyway.
+	 *
+	 * Furthermore, there is a bug in the linux kernel that causes a hang in
+	 * the fork() system call used by the crash handler to exec() a debugger,
+	 * and this may be due to signal delivery.
+	 *
+	 * Calling crash_abort() will ensure synchronous crash handling.
+	 *		--RAM, 2011-10-24
+	 */
+
+	crash_abort();
+
+#undef STACK_OFF
+}
+
+/*
+ * Trace the code path leading to this assertion.
+ */
+static NO_INLINE void G_GNUC_COLD
+assertion_stacktrace(void)
+{
+#define STACK_OFF	2		/* 2 extra calls: assertion_warning(), then here */
+
+	stacktrace_where_safe_print_offset(STDERR_FILENO, STACK_OFF);
+	if (log_stdout_is_distinct())
+		stacktrace_where_safe_print_offset(STDOUT_FILENO, STACK_OFF);
+
+#undef STACK_OFF
+}
+
+/*
+ * Due to an optimizer bug in gcc 4.2.1 (and maybe later verions), avoid
+ * specifying the REGPARM(1) attribute in the assertion_xxx() routines
+ * or the pointer being passed will be garbage, causing a segmentation fault
+ * in assertion_message().
+ *		--RAM, 2009-10-31
+ */
+
+NO_INLINE void G_GNUC_COLD
+assertion_warning(const assertion_data * const data)
+{
+	assertion_message(data, FALSE);
+	assertion_stacktrace();
+}
+
+NO_INLINE void G_GNUC_COLD
+assertion_warning_log(const assertion_data * const data,
+	const char * const fmt, ...)
+{
+	static str_t *str;
+	va_list args;
+
+	assertion_message(data, TRUE);
+
+	if G_UNLIKELY(NULL == str)
+		str = str_new_not_leaking(512);
+
+	/*
+	 * Log additional message.
+	 */
+
+	va_start(args, fmt);
+	str_vprintf(str, fmt, args);
+	va_end(args);
+
+	{
+		char time_buf[18];
+		DECLARE_STR(4);
+
+		crash_time(time_buf, sizeof time_buf);
+
+		print_str(time_buf);
+		print_str(" (WARNING): ");
+		print_str(str_2c(str));
+		print_str("\n");
+		flush_err_str();
+		if (log_stdout_is_distinct())
+			flush_str(STDOUT_FILENO);
+	}
+
+	assertion_stacktrace();
+}
+
+NO_INLINE void G_GNUC_COLD
+assertion_failure(const assertion_data * const data)
+{
+	assertion_message(data, TRUE);
+
+	/*
+	 * Record the root cause of the assertion failure to be able to log it
+	 * in the crash log in case they don't have gdb available.
+	 */
+
+	crash_assert_failure(data);
+	assertion_abort();
+}
+
+NO_INLINE void G_GNUC_COLD
+assertion_failure_log(const assertion_data * const data,
+	const char * const fmt, ...)
+{
+	va_list args;
+	const char *msg;
+
+	assertion_message(data, TRUE);
+
+	/*
+	 * Record the root cause of the assertion failure to be able to log it
+	 * in the crash log in case they don't have gdb available.
+	 */
+
+	crash_assert_failure(data);
+
+	/*
+	 * Record additional message in the crash log as well.
+	 */
+
+	va_start(args, fmt);
+	msg = crash_assert_logv(fmt, args);
+	va_end(args);
+
+	/*
+	 * Log additional message.
+	 */
+
+	if (msg != NULL) {
+		char time_buf[18];
+		DECLARE_STR(4);
+
+		crash_time(time_buf, sizeof time_buf);
+
+		print_str(time_buf);
+		print_str(" (FATAL): ");
+		print_str(msg);
+		print_str("\n");
+		flush_err_str();
+		if (log_stdout_is_distinct())
+			flush_str(STDOUT_FILENO);
+	}
+
+	assertion_abort();
 }
 
 /* vi: set ts=4 sw=4 cindent: */
