@@ -37,6 +37,14 @@
 
 #include "common.h"
 
+/**
+ * Callback for special allocation of the hash table object.
+ *
+ * @param allocator		the object handling allocation (opaque)
+ * @param len			length of the block to allocate
+ */
+typedef void *(*hash_table_alloc_t)(void *allocator, size_t len);
+
 typedef struct hash_table hash_table_t;
 
 typedef void (*hash_table_foreach_func)(
@@ -47,7 +55,17 @@ typedef gboolean (*hash_table_eq_func)(const void *a, const void *b);
 hash_table_t *hash_table_new(void);
 hash_table_t *hash_table_new_full(
 	hash_table_hash_func hash, hash_table_eq_func eq);
+void hash_table_clear(hash_table_t *ht);
 void hash_table_destroy(hash_table_t *ht);
+void hash_table_destroy_null(hash_table_t **ht_ptr);
+
+void hash_table_readonly(hash_table_t *ht);
+void hash_table_writable(hash_table_t *ht);
+
+hash_table_t *hash_table_new_special(const hash_table_alloc_t alloc, void *obj);
+hash_table_t *hash_table_new_special_full(
+	const hash_table_alloc_t alloc, void *obj,
+	hash_table_hash_func hash, hash_table_eq_func eq);
 
 #if defined(MALLOC_SOURCE) || defined(VMM_SOURCE)
 /* These routines are reserved for the tracking malloc code */
@@ -58,13 +76,15 @@ void hash_table_destroy_real(hash_table_t *ht);
 #endif /* MALLOC_SOURCE || VMM_SOURCE */
 
 size_t hash_table_size(const hash_table_t *ht);
-size_t hash_table_memory_size(const hash_table_t *ht);
+size_t hash_table_memory(const hash_table_t *ht);
+size_t hash_table_arena_memory(const hash_table_t *ht);
 gboolean hash_table_insert(hash_table_t *ht,
 	const void *key, const void *value);
 void hash_table_replace(hash_table_t *ht, const void *key, const void *value);
 void *hash_table_lookup(const hash_table_t *ht, const void *key);
 gboolean hash_table_lookup_extended(const hash_table_t *ht,
 	const void *key, const void **kp, void **vp);
+gboolean hash_table_contains(const hash_table_t *ht, const void *key);
 gboolean hash_table_remove(hash_table_t *ht, const void *key);
 void hash_table_foreach(hash_table_t *ht, hash_table_foreach_func func,
 		void *data);
