@@ -381,7 +381,7 @@ event_set_mask_with_kqueue(struct poll_ctx *ctx, int fd,
 	}
 
 	if (-1 == (ret = kevent(ctx->master_fd, kev, i, NULL, 0, &zero_ts)))
-		g_error("kevent() failed: %s", g_strerror(errno));
+		s_error("kevent() failed: %m");
 
 	return ret;
 }
@@ -506,7 +506,7 @@ collect_events_with_devpoll(struct poll_ctx *ctx, int timeout_ms)
 
 	ret = ioctl(ctx->master_fd, DP_POLL, &dvp);
 	if (-1 == ret && !is_temporary_error(errno)) {
-		g_warning("check_dev_poll(): ioctl() failed: %s", g_strerror(errno));
+		g_warning("check_dev_poll(): ioctl() failed: %m");
 	}
 	return ret;
 }
@@ -615,7 +615,7 @@ collect_events_with_select(struct poll_ctx *ctx, int timeout_ms)
 
 	if (ret < 0) {
 		if (!is_temporary_error(errno)) {
-			g_warning("select() failed: %s", g_strerror(errno));
+			g_warning("select() failed: %m");
 		}
 		return -1;
 	}
@@ -646,7 +646,7 @@ collect_events_with_poll(struct poll_ctx *ctx, int timeout_ms)
 
 	ret = compat_poll(ctx->pfd_arr, ctx->max_poll_idx, timeout_ms);
 	if (-1 == ret && !is_temporary_error(errno)) {
-		g_warning("collect_events(): poll() failed: %s", g_strerror(errno));
+		g_warning("collect_events(): poll() failed: %m");
 	}
 	return ret;
 }
@@ -819,8 +819,7 @@ inputevt_timer(struct poll_ctx *ctx)
 
 	num_events = (*ctx->event_check_all)(ctx);
 	if (-1 == num_events && !is_temporary_error(errno)) {
-		g_warning("event_check_all(%d) failed: %s",
-			ctx->master_fd, g_strerror(errno));
+		g_warning("event_check_all(%d) failed: %m", ctx->master_fd);
 	}
 
 	ctx->dispatching = TRUE;
@@ -959,8 +958,7 @@ poll_func(GPollFD *gfds, unsigned n, int timeout_ms)
 
 #ifdef INPUTEVT_DEBUGGING
 	if (-1 == r) {
-		g_warning("INPUTEVT default poll function failed: %s",
-			g_strerror(errno));
+		g_warning("INPUTEVT default poll function failed: %m");
 	}
 #endif
 
@@ -1033,8 +1031,7 @@ inputevt_remove(unsigned *id_ptr)
 		(rl->writers ? INPUT_EVENT_W : 0);
 
 	if (-1 == (*ctx->event_set_mask)(ctx, fd, old, cur)) {
-		g_warning("event_set_mask(%d, %d) failed: %s",
-				ctx->master_fd, fd, g_strerror(errno));
+		g_warning("event_set_mask(%d, %d) failed: %m", ctx->master_fd, fd);
 	}
 
 	/* Mark as removed */
@@ -1188,8 +1185,8 @@ inputevt_add_source(inputevt_relay_t *relay)
 		(-1 == (*ctx->event_set_mask)(ctx, relay->fd,
 									 old, (old | relay->condition))
 	) {
-		g_error("event_set_mask(%d, %d, ...) failed: %s",
-			ctx->master_fd, relay->fd, g_strerror(errno));
+		g_error("event_set_mask(%d, %d, ...) failed: %m",
+			ctx->master_fd, relay->fd);
 	}
 
 	g_assert(0 != id);	
@@ -1222,7 +1219,7 @@ init_with_kqueue(struct poll_ctx *ctx)
 	const int fd = kqueue();
 	
 	if (!is_valid_fd(fd)) {
-		g_warning("kqueue() failed: %s", g_strerror(errno));
+		g_warning("kqueue() failed: %m");
 		return -1;
 	}
 
@@ -1250,7 +1247,7 @@ init_with_devpoll(struct poll_ctx *ctx)
 	const int fd = get_non_stdio_fd(open("/dev/poll", O_RDWR));
 
 	if (!is_valid_fd(fd)) {
-		g_warning("open(\"/dev/poll\", O_RDWR) failed: %s", g_strerror(errno));
+		g_warning("open(\"/dev/poll\", O_RDWR) failed: %m");
 		return -1;
 	}
 
@@ -1278,7 +1275,7 @@ init_with_epoll(struct poll_ctx *ctx)
 	const int fd = epoll_create(1024 /* Just an arbitrary value as hint */);
 
 	if (!is_valid_fd(fd)) {
-		g_warning("epoll_create() failed: %s", g_strerror(errno));
+		g_warning("epoll_create() failed: %m");
 		return -1;
 	}
 
