@@ -35,11 +35,12 @@
 
 #include "signal.h"
 #include "ckalloc.h"
+#include "crash.h"
+#include "glib-missing.h"       /* For g_strlcpy() */
 #include "log.h"
 #include "misc.h"
-#include "crash.h"
+#include "str.h"
 #include "unsigned.h"
-#include "glib-missing.h"       /* For g_strlcpy() */
 
 #include "override.h"	/* Must be the last header included */
 
@@ -221,6 +222,36 @@ signal_trampoline(int signo)
 	}
 }
 
+#ifdef SA_SIGINFO
+/**
+ * Extended trapping for harmful signals for which we can gather extra
+ * information from the siginfo_t structure.
+ */
+static void
+signal_trampoline_extended(int signo, siginfo_t *si, void *u)
+{
+	(void) u;
+
+	/*
+	 * Log faulting address and propagate that information in the
+	 * crash log as an error message.
+	 */
+
+	in_signal_handler++;
+	{
+		char data[80];
+
+		str_bprintf(data, sizeof data, "got %s for VA=%p",
+			signal_name(signo), si->si_addr);
+		s_critical("%s", data);
+		crash_set_error(data);
+	}
+	in_signal_handler--;
+
+	signal_trampoline(signo);
+}
+#endif	/* SA_SIGINFO */
+
 /**
  * Installs a signal handler.
  *
@@ -264,15 +295,29 @@ signal_set(int signo, signal_handler_t handler)
 		struct sigaction sa, osa;
 		
 		sa = zero_sa;
-		sa.sa_handler = trampoline;
 		sigemptyset(&sa.sa_mask);
 		sa.sa_flags = signo != SIGALRM ? SA_RESTART : 0;
 
 		switch (signo) {
 #ifdef SIGBUS
 		case SIGBUS:
+#endif
+#ifdef SIGTRAP
+		case SIGTRAP:
 #endif
 		case SIGSEGV:
+		case SIGFPE:
+		case SIGILL:
+#ifdef SA_SIGINFO
+			if (signal_trampoline == trampoline) {
+				sa.sa_flags |= SA_SIGINFO;
+				sa.sa_sigaction = signal_trampoline_extended;
+			} else {
+				sa.sa_handler = trampoline;
+			}
+#else
+			sa.sa_handler = trampoline;
+#endif
 #ifdef SA_NODEFER
 			sa.sa_flags |= SA_NODEFER;
 #endif
@@ -280,6 +325,9 @@ signal_set(int signo, signal_handler_t handler)
 			sa.sa_flags |= SA_RESETHAND;
 #endif
 			break;
+		default:
+			sa.sa_handler = trampoline;
+			break;
 		}
 
 		ret = sigaction(signo, &sa, &osa) ? SIG_ERR : osa.sa_handler;
@@ -384,7 +432,7 @@ signal_leave_critical(const sigset_t *oset)
 #ifdef HAS_SIGPROCMASK
 	if (!in_critical_section) {
 		if (-1 == sigprocmask(SIG_SETMASK, oset, NULL))
-			s_error("cannot leave critical section: %s", g_strerror(errno));
+			s_error("cannot leave critical section: %m");
 	}
 #else
 	(void) oset;
@@ -404,7 +452,14 @@ signal_init(void)
 			signal_handler[i] = SIG_DFL;	/* Can't assume it's NULL */
 		}
 
-		sig_chunk = ck_init(SIGNAL_CHUNK_SIZE, SIGNAL_CHUNK_RESERVE);
+		/* 
+		 * Chunk allocated as non-leaking because the signal chunk must
+		 * remain active up to the very end, way past the point where we're
+		 * supposed to have freed everything and leak detection kicks in.
+		 */
+
+		sig_chunk =
+			ck_init_not_leaking(SIGNAL_CHUNK_SIZE, SIGNAL_CHUNK_RESERVE);
 	}
 
 	g_assert(sig_chunk != NULL);	/* We're initialized now */
@@ -416,7 +471,7 @@ signal_init(void)
 void
 signal_close(void)
 {
-	ck_destroy_null(&sig_chunk);
+	/* Nothing to do */
 }
 
 /* vi: set ts=4 sw=4 cindent:  */
