@@ -42,6 +42,7 @@
 
 #define T_ACQL  FOURCC_NATIVE('A','C','Q','L')
 #define T_ACQX  FOURCC_NATIVE('A','C','Q','X')
+#define T_AGIO  FOURCC_NATIVE('A','G','I','O')
 #define T_AGNT  FOURCC_NATIVE('A','G','N','T')
 #define T_ARES  FOURCC_NATIVE('A','R','E','S')
 #define T_ATOM  FOURCC_NATIVE('A','T','O','M')
@@ -100,6 +101,11 @@
 #define T_RASP  FOURCC_NATIVE('R','A','S','P')
 #define T_RAZA	FOURCC_NATIVE('R','A','Z','A')
 #define T_RAZB	FOURCC_NATIVE('R','A','Z','B')
+#define T_RAZL	FOURCC_NATIVE('R','A','Z','L')
+#define T_RZCA	FOURCC_NATIVE('R','Z','C','A')
+#define T_RZCB	FOURCC_NATIVE('R','Z','C','B')
+#define T_RZCC	FOURCC_NATIVE('R','Z','C','C')
+#define T_SHLN	FOURCC_NATIVE('S','H','L','N')
 #define T_SHNB	FOURCC_NATIVE('S','H','N','B')
 #define T_SNOW	FOURCC_NATIVE('S','N','O','W')
 #define T_SNUT	FOURCC_NATIVE('S','N','U','T')
